"""
dict 관련 유틸
"""
import json
from decimal import Decimal


def convert_number_to_decimal(dict_detail):
    for key, value in dict_detail.items():
        if isinstance(value, float):
            dict_detail[key] = Decimal(str(value))
        elif isinstance(dict_detail[key], dict):
            convert_number_to_decimal(dict_detail[key])
        elif isinstance(dict_detail[key], list):
            for item in dict_detail[key]:
                if isinstance(item, dict):
                    convert_number_to_decimal(item)
    return dict_detail


def convert_decimal_to_number(dict_detail):
    for key, value in dict_detail.items():
        if isinstance(value, Decimal):
            if value % 1 == 0:
                dict_detail[key] = int(value)
            else:
                dict_detail[key] = float(value)
        elif isinstance(dict_detail[key], dict):
            convert_decimal_to_number(dict_detail[key])
        elif isinstance(dict_detail[key], list):
            for item in dict_detail[key]:
                if isinstance(item, dict):
                    convert_decimal_to_number(item)
    return dict_detail



def _convert_dynamodb_json(dynamodb_json):
    if isinstance(dynamodb_json, dict):
        if 'S' in dynamodb_json:
            return dynamodb_json['S']
        elif 'N' in dynamodb_json:
            return float(dynamodb_json['N']) if '.' in dynamodb_json['N'] else int(dynamodb_json['N'])
        elif 'M' in dynamodb_json:
            return {k: _convert_dynamodb_json(v) for k, v in dynamodb_json['M'].items()}
        elif 'L' in dynamodb_json:
            return [_convert_dynamodb_json(v) for v in dynamodb_json['L']]
        elif 'BOOL' in dynamodb_json:
            return dynamodb_json['BOOL']
        elif 'NULL' in dynamodb_json:
            return None
    return dynamodb_json


def convert_dynamodb_document_to_dict(document: dict):
    return {key: _convert_dynamodb_json(value) for key, value in document.items()}


if __name__ == '__main__':
    # r = get_value_by_dot_representation({'a': {'b': {'c': {'d': 'e'}}}}, 'a')
    # print(r)
    print(convert_number_to_decimal({
        'd': {
            'd': {
                '1': 14.232,
                'd': 'ss'
            },
            's': {
                'g': 'sss'
            },
            'ss': [
                {
                    'ok': 1.141,
                    'ss': 'ssss'
                }
            ]
        }
    }))


import boto3
from typing import List


class SSM:
    def __init__(self, boto3_session, region_name='us-east-1'):
        self.client = boto3_session.client('ssm', region_name=region_name)
        self.region = boto3_session.region_name

    def run_commands(self, instance_id:str, working_directory:str, command: str):
        response = self.client.send_command(
            InstanceIds=[instance_id],
            DocumentName='AWS-RunShellScript',
            Parameters={"workingDirectory":[working_directory],"executionTimeout":["3600"], 'commands': [command]},
            Targets=[{"Key":"InstanceIds","Values":[instance_id]}]

        )
        command_id = response['Command']['CommandId']
        output = self.client.get_command_invocation(
            CommandId=command_id,
            InstanceId=instance_id
        )
        return output



"""
각 함수에 데코레이터를 적용하여 특정 경로로 라우팅되도록 만들기 위해 데코레이터를 정의합니다.
"""

# controller_decorator.py
controller_registry = {}

def controller(module_name: str):
    def decorator(func):
        controller_registry[module_name + '.' + func.__name__] = func
        return func
    return decorator



import time

from sawsi.shared import error_util
from sawsi.shared import handler_util
from sawsi.handler.router import router
import errs


# 아래 핸들러는 share.error_util.AppError 에러가 발생할시에, 자동으로
# 에러를 response 객체에 담아 코드와 메시지로 구분하여 전송합니다.
@handler_util.aws_handler_wrapper(
    error_receiver=lambda errmsg: print(errmsg),  # 이 Lambda 함수를 슬랙 Webhook 등으로 대체하면 에러 발생시 모니터링이 가능합니다.
    content_type='application/json',  # 기본적으로 JSON 타입을 반환합니다.
    use_traceback=True,  # 에러 발생시 상세 값을 응답에 전달할지 유무입니다.
    ignore_app_errors=[
        errs.no_session
    ]
)
def handler(event, context):
    """
    AWS LAMBDA에서 API Gateway 를 통해 콜한 경우
    """
    # API Gateway 로부터 Lambda 에 요청이 들어오면 다음과 같이 body 와 headers 를 분리하여 dict 형태로 반환합니다.
    body = handler_util.get_body(event, context)
    headers = handler_util.get_headers(event, context)
    source_ip = handler_util.get_source_ip(event, context)

    # 헤더로부터 세션 가져오기
    session_id = headers.get('session_id', None)

    # 아래부터는 사용자가 직접 응용하여 핸들러를 구성, 다른 함수들로 라우팅합니다.
    cmd = body.get('cmd', None)

    # user 값은 무조건 예약해서 사용해야, body 에 임의로 값을 넣어서 조작할 수 없음.
    body['user'] = None
    if session_id:
        # 주석 내용은 프로그램 내용에 따라 변경할것
        # body['user'] = some_function_to_get_user(session_id)
        pass

    return router(cmd, body, headers)



from sawsi.shared import error_util
from sawsi.shared import handler_util
from sawsi.shared.function_util import try_run
from concurrent.futures import ThreadPoolExecutor
import json


# 아래 핸들러는 share.error_util.AppError 에러가 발생할시에, 자동으로
# 에러를 response 객체에 담아 코드와 메시지로 구분하여 전송합니다.
@handler_util.aws_handler_wrapper(
    error_receiver=lambda errmsg: print(errmsg),  # 이 Lambda 함수를 슬랙 Webhook 등으로 대체하면 에러 발생시 모니터링이 가능합니다.
    content_type='application/json',  # 기본적으로 JSON 타입을 반환합니다.
    use_traceback=True,  # 에러 발생시 상세 값을 응답에 전달할지 유무입니다.
)
def handler(event, context):
    """
    AWS LAMBDA에서 SQS 를 통해 트리거된 경우
    """
    records = event.get('Records', [])
    futures = []
    with ThreadPoolExecutor(max_workers=4) as worker:
        for record in records:
            futures.append(worker.submit(try_run, route_record, record))

    # futures 까보면서 에러 있는거 있으면 전송
    for future in futures:
        result, exception, traceback = future.result()
        # 아래 주석을 해제하면 실행 중 에러가 날시에 SQS에서 재시도를 합니다. 신중하게 처리해주세요.
        # if exception:
        #     raise Exception(f'{exception}\n{traceback}')

    return {}


def route_record(record):
    message_body = record.get('body', '{}')
    message_body = json.loads(message_body)
    body = message_body.get('body', {})
    body_type = message_body.get('type', None)
    # SQS 레코드를 읽어서 작업을 수행 합니다.
    # SQS 는 api.py 파일의 sqs 인스턴스의 def send_message(self, message_body) 함수를 콜 하여 콜 할 수 있습니다.
    # 재귀 호출로 인해 비용이 비정상적으로 발생할 수 있음을 유의하세요.

    if body_type == '{{SQS를 콜한 측에서 설정한 body_type}}':
        # TODO YOU SHOULD IMP HERE
        # TASK(body)
        return

    raise error_util.NO_SUCH_BODY_TYPE_BY_SQS


import boto3


class SQS:
    def __init__(self, boto3_session, region_name='us-east-1'):
        self.client = boto3_session.client('sqs', region_name=region_name)
        self.region = boto3_session.region_name


    def send_message(self, queue_url, message_body):
        response = self.client.send_message(
            QueueUrl=queue_url,
            MessageBody=message_body
        )
        return response
